import { dbconn } from '../decorators/dbconn'
import { getModelForClass, index, modelOptions, prop, ReturnModelType } from '@typegoose/typegoose'
import { BaseModule } from './Base'
import { customAlphabet } from 'nanoid'
const nanoid = customAlphabet('2345678abcdefghjkmnpqrstwxy', 32)

const jsonExcludeKeys = ['updatedAt', '__v', '_id', 'createdAt', 'isBot', 'type']

/**
 * 用于存储排行榜的用户信息
 * 从redis中获取accountId和积分, 然后从这里获取其他信息
 */
@dbconn()
@index({ accountId: 1 }, { unique: true })
@index({ gameId: 1, channelId: 1 }, { unique: false })
@modelOptions({
  schemaOptions: { collection: 'rank_user', timestamps: true },
})
class RankUserClass extends BaseModule {
  @prop()
  accountId: string
  @prop()
  gameId: string
  @prop()
  channelId: string
  @prop({ default: false })
  isBot: boolean
  @prop()
  nickname: string
  @prop()
  avatar: string

  @prop()
  rankTitle: string
  @prop()
  extInfo: string
  /**
   * @type {number} 0: 只在总榜有, 1: 在周榜和总榜都有
   */
  @prop({ default: 1 })
  type: number

  public static async getByAccountID(this: ReturnModelType<typeof RankUserClass>, accountId: string) {
    let records = await this.find({ accountId }).limit(1)
    return records.length > 0 ? records[0] : null
  }

  public static async userMapByAccountIDS(this: ReturnModelType<typeof RankUserClass>, accountIds: string[]) {
    const records = await this.find({ accountId: { $in: accountIds } })
    let map: Map<string, any> = new Map()
    for (let record of records) {
      map.set(record.accountId, record.toJson())
    }
    return map
  }

  public static async parseRankList(this: ReturnModelType<typeof RankUserClass>, datas: any[]) {
    let rankList: { accountId: string; score: any }[] = []
    let accountIDS: string[] = []
    for (let i = 0, l = datas.length; i < l; i += 2) {
      rankList.push({ accountId: datas[i] as string, score: datas[i + 1] << 0 })
      accountIDS.push(datas[i])
    }
    const userMap = await RankUser.userMapByAccountIDS(accountIDS)
    for (let d of rankList) {
      if (userMap.has(d.accountId)) {
        Object.assign(d, userMap.get(d.accountId))
      }
    }
    return rankList
  }

  public static randomAccountId(gameId: string, channelId: string) {
    return `${channelId}_${gameId}_${nanoid()}`
  }

  public toJson(): any {
    let result: any = {}
    // @ts-ignore
    for (let key in this._doc) {
      if (jsonExcludeKeys.indexOf(key) == -1) {
        result[key] = this[key]
      }
    }
    return result
  }
}

export const RankUser = getModelForClass(RankUserClass, { existingConnection: RankUserClass.db })
