import { singleton } from '../decorators/singleton'
import * as schedule from 'node-schedule'
import logger from '../logger/logger'
import { RankUser } from '../models/RankUser'
import { delRankList, rankKey, updateRank } from '../services/rank.svr'
import { fetchRankCfg, randomUsers } from '../services/jcfw.svr'
import { getThisWeekData } from '../utils/time.util'

// 每天晚上1点更新
const WORLD_RANK_TASK = '1 0 1 * * *'
// 总榜机器人总数
const MAX_ROBOT_COUNT = 40
@singleton
export default class RankSchedule {
  /**
   * 取出原来的机器人列表
   * 移除n个, 同时补充n个机器人
   * 补充的n个机器人更新至总榜
   */
  async initRankData({ gameId, channelId, refresh }: { gameId: string; channelId: string; refresh?: boolean }) {
    let oldList = await RankUser.find({ gameId, channelId, isBot: true, type: 1 })
    const { startDay } = getThisWeekData()
    const today = new Date().format('yyyy-M-d')
    const needRest = today === startDay || refresh
    if (oldList.length > 0) {
      const maxRemoveCount = needRest ? Math.random2(5, 10) | 0 : Math.random2(2, 6) | 0
      let deleteCount = 0
      let tmpList = []
      for (let i = 0, l = oldList.length; i < l; i++) {
        let record = oldList[i]
        if (Math.random() > 0.5 && deleteCount < maxRemoveCount) {
          record.type = 0
          await record.save()
          deleteCount++
        } else {
          tmpList.push(record)
        }
      }
      oldList = tmpList
    }
    let needCount = MAX_ROBOT_COUNT - oldList.length
    try {
      let users = await randomUsers('', needCount)
      for (const user of users) {
        let accountId = RankUser.randomAccountId(gameId, channelId)
        let record = await RankUser.insertOrUpdate(
          { gameId, channelId, accountId },
          { nickname: user.nickname, avatar: user.avatar_url, isBot: true, type: 1 },
        )
        oldList.push(record)
      }
    } catch (err) {
      logger.error('error get random user', err)
    }

    let weekKey = rankKey(gameId, channelId, false)
    if (needRest) {
      await delRankList(weekKey)
    }

    let totalKey = rankKey(gameId, channelId, true)
    const { min, max, valType } = await fetchRankCfg(gameId, channelId)
    for (let record of oldList) {
      try {
        let score = Math.random2(min, max) | 0
        await updateRank(record.accountId, score, weekKey, valType)
        await updateRank(record.accountId, score, totalKey, valType)
      } catch (err) {
        logger.error('update exists rank error')
      }
    }
  }
  async parseAllRecord() {
    logger.info('=================begin update world rank.=================')
    let gameList = []
    try {
      gameList = await RankUser.aggregate([
        { $match: { gameId: { $exists: true }, channelId: { $exists: true } } },
        { $group: { _id: { gameId: '$gameId', channelId: '$channelId' } } },
      ])
    } catch (err) {
      logger.error('err get game list for update world rank schedule', err)
    }
    for (const game of gameList) {
      try {
        await this.initRankData({ gameId: game._id.gameId, channelId: game._id.channelId })
      } catch (err) {
        logger.error(`err init game world rank, ${game}`, err)
      }
    }
    logger.info('=================end update world rank.=================')
  }
  scheduleAll() {
    logger.log('已添加更新世界排行榜的定时任务')
    const job = schedule.scheduleJob(WORLD_RANK_TASK, async () => {
      await this.parseAllRecord()
    })
  }
}
