import { RedisClient } from '../redis/RedisClient'
import crypto from 'crypto'
import axios from 'axios'
import { GameCfgCache } from '../cache/GameCfgCache'

/**
 * 获取某游戏的所有客户端配置
 * @param {string} gameId
 * @param {string} channel
 * @return {Promise<{key: string, value: string}[]>}
 */
export async function getGameClientCfg(gameId: string, channel: string) {
  const key = `config:${gameId}:${channel}`
  const records = await new RedisClient().get(key)
  let results: { key: string; value: string }[] = []
  if (records) {
    results = JSON.parse(records as string)
  }
  return results
}

export async function getGameSvrCfg(gameId: string, channel: string) {
  const key = `server_config:${gameId}:${channel}`
  const records = await new RedisClient().get(key)
  let results: { key: string; value: string }[] = []
  if (records) {
    results = JSON.parse(records as string)
  }
  return results
}

/**
 * 获取游戏的订阅消息配置
 * @param {string} gameId
 * @param {string} channel
 * @param svr 是否是服务端配置
 * @return {Promise<any>}
 */
export async function getSubscribeCfg(gameId: string, channel: string, svr: boolean = false) {
  const cfgs = svr ? await getGameSvrCfg(gameId, channel) : await getGameClientCfg(gameId, channel)
  const result: any = {}
  for (let i = 0, l = cfgs.length; i < l; i++) {
    if (cfgs[i].key === 'subscribe_cfg') {
      const val = JSON.parse(cfgs[i].value)
      Object.assign(result, val)
      break
    }
  }
  return result
}

/**
 * 根据type获取对应的订阅消息模板id
 * @param {string} gameId
 * @param {string} channel
 * @param {string} type
 * @param {string} templateId
 * @return {Promise<{type: string, templateId: string}>}
 */
export async function getSubscribeInfo({
  gameId,
  channel,
  type,
  templateId,
}: {
  gameId: string
  channel: string
  type?: string
  templateId?: string
}) {
  const cfg = await getSubscribeCfg(gameId, channel)
  if (type) {
    templateId = cfg[type]
  } else if (templateId) {
    for (let key of Object.keys(cfg)) {
      if (cfg[key] === templateId) {
        type = key
        break
      }
    }
  }
  return { type, templateId }
}

/**
 * 获取订阅消息的消息模板
 * @param {string} gameId
 * @param {string} channel
 * @param {string} type
 */
export async function getSubscribeMsg(gameId: string, channel: string, type: string) {
  const cfg = await getSubscribeCfg(gameId, channel, true)
  const data = cfg[type]
  if (data) {
    for (let key of Object.keys(data)) {
      if (data[key].indexOf('$date') > -1) {
        data[key] = { value: data[key].replace('$date', new Date().format('yyyy-MM-dd')) }
      } else {
        data[key] = { value: data[key] }
      }
    }
  }
  return data
}

function createSign(secretKey: string, paramStr: string, timestamp: number) {
  paramStr = `${paramStr}:${timestamp}${secretKey}`
  return crypto.createHash('md5').update(paramStr, 'utf8').digest('hex')
}

/**
 * 获取随机用户
 * @param {string} accountId
 * @param {number} num
 */
export async function randomUsers(accountId: string, num: number) {
  num = num || 10
  const excludeAccountids = accountId ? `[${accountId}]` : '[]'
  const timestamp = new Date().getTime()
  const url = 'https://service.kingsome.cn/webapp/index.php?c=Voodoo&a=getRobotList'
  const paramStr = `exclude_accountids=${excludeAccountids}&exclude_names=[]&num=${num}`
  const signStr = createSign('70e32abc60367adccaa9eb7b56ed821b', paramStr, timestamp)
  const link = `${url}&${paramStr}&sign=${signStr}&timestamp=${timestamp}`
  const { data } = await axios.get(link)
  if (data.errcode === 0) {
    return data.robot_list
  } else {
    return []
  }
}
const RANK_MAX_KEY = 'world_rank_max'
const RANK_MIN_KEY = 'world_rank_min'
const RANK_TYPE_KEY = 'world_rank_type'
const RANK_VALTYPE_KEY = 'world_rank_valtype'
/**
 * 获取世界排行榜的配置
 * @param {string} gameId
 * @param {string} channel
 * @return min 生成排行榜的最小值
 * @return max 生成排行榜的最大值
 * @return type 排行榜类型, 0: 周榜, 1: 总榜
 * @return valType: 排行榜值存储类型, 0: 存最大值, 1: 存最新值
 */
export async function fetchRankCfg(gameId: string, channel: string) {
  let cfgs = new GameCfgCache().getCfg(gameId, channel)
  if (!cfgs) {
    cfgs = await getGameSvrCfg(gameId, channel)
    new GameCfgCache().updateCfg(gameId, channel, cfgs)
  }
  let type = 0
  let max = 1000
  let min = 500
  let valType = 0
  for (let i = 0, l = cfgs.length; i < l; i++) {
    const cfg = cfgs[i]
    switch (cfg.key) {
      case RANK_MAX_KEY:
        max = +cfg.value
        break
      case RANK_MIN_KEY:
        min = +cfg.value
        break
      case RANK_TYPE_KEY:
        type = +cfg.value
        break
      case RANK_VALTYPE_KEY:
        valType = +cfg.value
        break
    }
  }
  return { min, max, type, valType }
}
