import { RedisClient } from '../redis/RedisClient'
import { RANK_SCORE } from '../constants/BaseConsts'

const MAX_TIME = 5000000000000

function generateRankKey(subKey?: string) {
  if (subKey) {
    return `${RANK_SCORE}_${subKey}`
  } else {
    return RANK_SCORE
  }
}

/**
 * 更新排行榜数据
 * @param subKey
 * @param {string} accountId
 * @param {number} score
 * @param valType 值存储类型, 0: 存最大值, 1: 存最新值
 */
export async function updateRank(accountId: string, score: number, subKey: string, valType: number) {
  let scoreL = parseFloat(`${score | 0}.${MAX_TIME - Date.now()}`)
  const key = generateRankKey(subKey)
  let scoreResult = score
  if (valType === 0) {
    let scoreOld = await getAccountScore(accountId, subKey)
    if (score > scoreOld) {
      await new RedisClient().zadd(key, scoreL, accountId)
    } else {
      scoreResult = scoreOld
    }
  } else {
    await new RedisClient().zadd(key, scoreL, accountId)
  }
  const countTotal = (await new RedisClient().zcard(key)) as number
  const countOver = (await new RedisClient().zcount(key, 0, scoreL)) as number
  const over = countTotal ? countOver / countTotal : 0.99
  return { scoreReal: scoreResult, over, total: countTotal }
}

/**
 * 获取指定用户的积分
 * @param {string} accountId
 * @param subKey
 */
export async function getAccountScore(accountId: string, subKey?: string) {
  let score = await new RedisClient().zscore(generateRankKey(subKey), accountId)
  if (score) {
    return +score << 0
  } else {
    return 0
  }
}

/**
 * 获取超越只
 * @param {string} accountId
 * @param {number} score
 * @param {string} subKey
 * @return {Promise<number>}
 */
export async function currentIdx(accountId: string, score: number, subKey?: string) {
  const key = generateRankKey(subKey)
  let countTotal = (await new RedisClient().zcard(key)) as number
  let scoreL = parseFloat(`${score | 0}.${MAX_TIME - Date.now()}`)
  let countOver = (await new RedisClient().zcount(key, 0, scoreL)) as number
  return countOver / countTotal
}

/**
 * 获取指定分数段的玩家
 * @param {number} min
 * @param {number} max
 * @param subKey
 */
export async function usersByScore(min: number, max: number, subKey?: string) {
  return await new RedisClient().zrangebyscore(generateRankKey(subKey), min, max)
}

/**
 * 获取指定用户的排名
 * @param {string} accountId
 * @param subKey
 */
export async function getAccountRank(accountId: string, subKey?: string) {
  return await new RedisClient().zrevrank(generateRankKey(subKey), accountId)
}

/**
 * 获取从start到end的玩家列表
 * @param {number} start
 * @param {number} end
 * @param subKey
 */
export async function getRankList(start: number, end: number, subKey?: string) {
  return await new RedisClient().zrevrange(generateRankKey(subKey), start, end)
}

/**
 * 获取某排行榜的成员总数
 * @param {string} subKey
 */
export async function getRankCount(subKey?: string) {
  return await new RedisClient().zcard(generateRankKey(subKey))
}

/**
 * 获取帐号附近的几个记录
 * @param {string} accountId
 * @param subKey
 */
export async function getRankNear(accountId: string, subKey?: string) {
  let rank: any = await getAccountRank(accountId)
  if (rank == null) {
    return []
  }
  let total: any = await new RedisClient().zcard(generateRankKey(subKey))
  let start = 0
  let count = 2
  let end = start + count
  if (rank > 0 && rank < total - 1) {
    start = rank - 1
    end = start + count
  } else if (rank == total - 1) {
    start = rank - 2
    end = total
  }
  start = start < 0 ? 0 : start
  end = end > total - 1 ? total - 1 : end
  let datas: any = await getRankList(start, end)
  let results: any[] = []
  for (let i = 0, l = datas.length; i < l; i += 2) {
    results.push({
      rank: start + i / 2,
      accountId: datas[i],
      score: datas[i + 1] << 0,
    })
  }
  return results
}

/**
 * 移除某个榜单
 * @param {string} subKey
 * @return {Promise<void>}
 */
export async function delRankList(subKey?: string) {
  await new RedisClient().del(generateRankKey(subKey))
}

export function rankKey(gameId: string, channelId: string, all: boolean) {
  return all ? `${gameId}_${channelId}_total` : `${gameId}_${channelId}_week`
}
