import axios, { AxiosRequestConfig } from 'axios'
import fs from 'fs'
import { WxTokenCache } from './WxTokenCache'

const URL_QRCODE = 'https://api.weixin.qq.com/wxa/getwxacodeunlimit'
const URL_TOKEN = 'https://api.weixin.qq.com/cgi-bin/token'
const URL_MSG_CHECK = 'https://api.weixin.qq.com/wxa/msg_sec_check'
const URL_SUBSCRIBE_MSG = 'https://api.weixin.qq.com/cgi-bin/message/subscribe/send'
const URL_GENERATE_URL_LINK = 'https://api.weixin.qq.com/wxa/generate_urllink'
const URL_GENERATE_SCHEMA = 'https://api.weixin.qq.com/wxa/generatescheme'

/**
 * 刷新微信api的access token
 * @param {string} appId
 * @param {string} appSecret
 * @param {boolean} refresh 是否强制刷新
 */
export async function refreshToken(appId: string, appSecret: string, refresh: boolean = false) {
  const util = new WxTokenCache()
  let token = util.getToken(appId)
  if (token && !refresh) {
    return token
  }
  const link = `${URL_TOKEN}?grant_type=client_credential&appid=${appId}&secret=${appSecret}`
  const { data } = await axios.get(link)
  if (!data.errcode) {
    util.updateToken(appId, data.access_token, data.expires_in)
    return data.access_token
  } else {
    throw new Error(data.errmsg)
  }
}

/**
 * 生成微信小程序码
 * @param {any} appId
 * @param {any} appSecret
 * @param {any} scene 用于额外的参数, 在客户端使用query.scene获取
 * @param {any} file 写入本地文件的路径
  // 在首次启动时通过 wx.getLaunchOptionsSync 接口获取
  const {query} = wx.getLaunchOptionsSync()
  const scene = decodeURIComponent(query.scene)

  // 或者在 wx.onShow 事件中获取
  wx.onShow(function ({query}) {
    // scene 需要使用 decodeURIComponent 才能获取到生成二维码时传入的 scene
    const scene = decodeURIComponent(query.scene)
  })
 */
export async function generateQr({ appId, appSecret, scene, file }) {
  const stream = fs.createWriteStream(file)
  const token = await refreshToken(appId, appSecret)
  const url = `${URL_QRCODE}?access_token=${token}`
  const reqParams = {
    scene: scene,
    width: 430,
    auto_color: false,
    line_color: { r: '0', g: '0', b: '0' },
  }
  let reqConfig: AxiosRequestConfig = {
    method: 'post',
    url,
    headers: {
      'Cache-Control': 'no-cache',
      'Content-Type': 'application/json',
    },
    responseType: 'stream',
    data: reqParams,
  }
  const { data } = await axios(reqConfig)
  data.pipe(stream)
  return true
}

/**
 * 微信小程序生成 URL Link
 * @param {any} appId
 * @param {any} appSecret
 */
export async function generateUrlLink({ appId, appSecret }) {
  const data = {}
  const token = await refreshToken(appId, appSecret)
  const url = `${URL_GENERATE_URL_LINK}?access_token=${token}`
  let reqConfig: AxiosRequestConfig = {
    method: 'post',
    url,
    headers: {
      'Cache-Control': 'no-cache',
      'Content-Type': 'application/json',
    },
    data,
  }
  return axios(reqConfig)
}

/**
 * 微信小程序生成 Schema
 * @param {any} appId
 * @param {any} appSecret
 */
export async function generateSchema({ appId, appSecret }) {
  const data = {}
  const token = await refreshToken(appId, appSecret)
  const url = `${URL_GENERATE_SCHEMA}?access_token=${token}`
  let reqConfig: AxiosRequestConfig = {
    method: 'post',
    url,
    headers: {
      'Cache-Control': 'no-cache',
      'Content-Type': 'application/json',
    },
    data,
  }
  return axios(reqConfig)
}

/**
 * 调用微信的security.msgSecCheck接口检查违法违规内容
 * @param {string} content
 */
export async function msgSecCheck(content: string) {
  let data = { content }
  const token = await refreshToken('wxf8c3da4e7dfe00a2', '8c0a1e88a6b43e4be80ed6a597c0b047')
  const url = `${URL_MSG_CHECK}?access_token=${token}`
  let reqConfig: AxiosRequestConfig = {
    method: 'post',
    url,
    headers: {
      'Cache-Control': 'no-cache',
      'Content-Type': 'application/json',
    },
    data,
  }
  return axios(reqConfig)
}

/**
 * 发送订阅消息
 * @param  appId
 * @param  appSecret
 * @param  openId       目标用户openid
 * @param  templateId
 * @param  data
 */
export async function sendWxSubscribeMessage({ appId, appSecret, openId, templateId, data }) {
  const params = { touser: openId, template_id: templateId, data }
  const token = await refreshToken(appId, appSecret)
  const url = `${URL_SUBSCRIBE_MSG}?access_token=${token}`
  let reqConfig: AxiosRequestConfig = {
    method: 'post',
    url,
    headers: {
      'Cache-Control': 'no-cache',
      'Content-Type': 'application/json',
    },
    data: JSON.stringify(params),
  }
  return axios(reqConfig)
}
