/**
 *
 * @param {Function} cb
 * @param {number} maxRetries
 * @param {any[]} errorWhiteList
 * @param {number} retries
 * @return {Promise<T>}
 */
export function retry<T = any>(cb: Function, maxRetries: number = 3, errorWhiteList: any[] = [], retries: number = 0) {
  return new Promise<T>((resolve, reject) => {
    cb()
      .then(resolve)
      .catch(e => {
        if (errorWhiteList.indexOf(e.constructor) !== -1 && retries++ < maxRetries) {
          setTimeout(() => {
            retry<T>(cb, maxRetries, errorWhiteList, retries)
              .then(resolve)
              .catch(e2 => reject(e2))
          }, Math.floor(Math.random() * Math.pow(2, retries) * 400))
        } else {
          reject(e)
        }
      })
  })
}

export class Deferred<T = any> {
  public promise: Promise<T>

  public resolve: Function
  public reject: Function

  constructor() {
    this.promise = new Promise<T>((resolve, reject) => {
      this.resolve = resolve
      this.reject = reject
    })
  }

  public then(func: (value: T) => any) {
    return this.promise.then.apply(this.promise, arguments)
  }

  public catch(func: (value: any) => any) {
    return this.promise.catch(func)
  }
}

/**
 * 简单限流的 Promise 队列
 * usage:
 const q = new PromiseQueue();
 [1, 2, 3, 4, 5, 6, 7, 8, 9, 10].forEach((v) => {
  q.add(
    () =>
      new Promise((resolve) => {
        setTimeout(() => {
          console.log(v);
          resolve();
        }, 1000);
      })
  );
});
 */
export class PromiseQueue {
  private readonly concurrency: number
  private _current: number
  private _list: any[]

  constructor(options: any = {}) {
    this.concurrency = options.concurrency || 2
    this._current = 0
    this._list = []
  }

  add(promiseFn) {
    this._list.push(promiseFn)
    this.loadNext()
  }

  loadNext() {
    if (this._list.length === 0 || this.concurrency === this._current) return

    this._current++
    const fn = this._list.shift()
    const promise = fn.call(this)
    promise.then(this.onLoaded.bind(this)).catch(this.onLoaded.bind(this))
  }

  onLoaded() {
    this._current--
    this.loadNext()
  }
}
